﻿'
' Copyright (c) 2009-2015, MindFusion LLC - Bulgaria.
'
Imports System.IO

Imports MindFusion.Scheduling

Imports DDay.iCal
Imports DDay.iCal.Components
Imports DDay.iCal.DataTypes
Imports DDay.iCal.Serialization
Imports DDay.iCal.Serialization.iCalendar.DataTypes


Partial Public Class _Default
	Inherits System.Web.UI.Page
	Private recurrentItems As Hashtable

	Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
		If Not IsPostBack Then
			calendar.[Date] = DateTime.Today.[Date]
			calendar.EndDate = DateTime.Today.[Date].AddMonths(2)

			AddScheduleData()
		End If
	End Sub

	Protected Sub btnLoad_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnLoad.Click
		Dim FileName As String = Server.MapPath(".") & "\calendar.ics"
		If Not File.Exists(FileName) Then
			Return
		End If
		LoadICalendar(FileName)
	End Sub

	Protected Sub btnSave_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSave.Click
		Dim FileName As String = Server.MapPath(".") & "\calendar.ics"
		SaveICalendar(FileName)
	End Sub

	Protected Sub btnClear_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnClear.Click
		calendar.Schedule.Clear()
		calendar.Selection.Reset()
	End Sub

	Private Function ToDaysOfWeek(ByVal d As DayOfWeek) As MindFusion.Scheduling.DaysOfWeek
		Select Case d

			Case DayOfWeek.Monday
				Return DaysOfWeek.Monday

			Case DayOfWeek.Tuesday
				Return DaysOfWeek.Tuesday

			Case DayOfWeek.Friday
				Return DaysOfWeek.Friday

			Case DayOfWeek.Saturday
				Return DaysOfWeek.Saturday

			Case DayOfWeek.Sunday
				Return DaysOfWeek.Sunday

			Case DayOfWeek.Wednesday
				Return DaysOfWeek.Wednesday

			Case DayOfWeek.Thursday
				Return DaysOfWeek.Thursday

		End Select

		Return DaysOfWeek.None
	End Function

	Private Function FromDaysOfWeek(ByVal d As DaysOfWeek) As DayOfWeek
		Select Case d

			Case DaysOfWeek.Monday
				Return DayOfWeek.Monday

			Case DaysOfWeek.Tuesday
				Return DayOfWeek.Tuesday

			Case DaysOfWeek.Friday
				Return DayOfWeek.Friday

			Case DaysOfWeek.Saturday
				Return DayOfWeek.Saturday

			Case DaysOfWeek.Sunday
				Return DayOfWeek.Sunday

			Case DaysOfWeek.Wednesday
				Return DayOfWeek.Wednesday

			Case DaysOfWeek.Thursday
				Return DayOfWeek.Thursday

		End Select

		Return DayOfWeek.Monday
	End Function

	Private Function ToDayOfWeekType(ByVal d As DayOfWeek) As DayOfWeekType
		Select Case d

			Case DayOfWeek.Monday
				Return DayOfWeekType.Monday

			Case DayOfWeek.Tuesday
				Return DayOfWeekType.Tuesday

			Case DayOfWeek.Friday
				Return DayOfWeekType.Friday

			Case DayOfWeek.Saturday
				Return DayOfWeekType.Saturday

			Case DayOfWeek.Sunday
				Return DayOfWeekType.Sunday

			Case DayOfWeek.Wednesday
				Return DayOfWeekType.Wednesday

			Case DayOfWeek.Thursday
				Return DayOfWeekType.Thursday

		End Select

		Return DayOfWeekType.Monday
	End Function

	Private Function FromDayOfWeekType(ByVal d As DayOfWeekType) As DayOfWeek
		Select Case d

			Case DayOfWeekType.Monday
				Return DayOfWeek.Monday

			Case DayOfWeekType.Tuesday
				Return DayOfWeek.Tuesday

			Case DayOfWeekType.Friday
				Return DayOfWeek.Friday

			Case DayOfWeekType.Saturday
				Return DayOfWeek.Saturday

			Case DayOfWeekType.Sunday
				Return DayOfWeek.Sunday

			Case DayOfWeekType.Wednesday
				Return DayOfWeek.Wednesday

			Case DayOfWeekType.Thursday
				Return DayOfWeek.Thursday

		End Select

		Return DayOfWeek.Monday
	End Function

	''' <summary>
	''' Add some contacts and locations to the schedule
	''' </summary>
	Private Sub AddScheduleData()
		Dim c As New Contact()
		c.FirstName = "P"
		c.LastName = "Sherman"
		calendar.Schedule.Contacts.Add(c)

		c = New Contact()
		c.FirstName = "Sherlock"
		c.LastName = "Holmes"
		calendar.Schedule.Contacts.Add(c)

		Dim l As New Location()
		l.Name = "Where is Nemo?"
		l.Country = "Australia"
		l.City = "Sydney"
		l.Address = "42, Wallaby Way"
		calendar.Schedule.Locations.Add(l)

		l = New Location()
		l.Name = "Marylebone"
		l.Country = "England"
		l.City = "London"
		l.Address = "221B, Baker Street"
		calendar.Schedule.Locations.Add(l)
	End Sub

	''' <summary>
	''' Obtains a location with the specified name.
	''' </summary>
	Private Function GetLocation(ByVal name As String) As Location
		For Each location As Location In calendar.Schedule.Locations
			If location.Name = name Then
				Return location
			End If
		Next

		Return Nothing
	End Function

	''' <summary>
	''' Obtains a contact with the specified name.
	''' </summary>
	Private Function GetContact(ByVal name As String) As Contact
		For Each contact As Contact In calendar.Schedule.Contacts
			If contact.Name = name Then
				Return contact
			End If
		Next

		Return Nothing
	End Function

	''' <summary>
	''' Registers the specified event as a master item of a recurrence.
	''' </summary>
	Private Sub RegisterRecurrenceMaster(ByVal id As String, ByVal item As Item)
		Dim list As List(Of Item) = Nothing
		If recurrentItems.ContainsKey(id) Then
			list = TryCast(recurrentItems(id), List(Of Item))
		Else
			list = New List(Of Item)()
			recurrentItems(id) = list
		End If

		list.Add(item)
	End Sub

	''' <summary>
	''' Copies all non-recurrent properties from the specified i-event
	''' to the specified MindFusion.Scheduling item.
	''' </summary>
	Private Sub CopyPropertiesNoRec(ByVal item As Item, ByVal ievent As [Event])
		' Transfer general properties
		item.StartTime = ievent.Start.Value
		item.EndTime = ievent.[End].Value
		item.AllDayEvent = ievent.IsAllDay

		If ievent.Summary IsNot Nothing Then
			item.HeaderText = ievent.Summary.Value
		End If

		If ievent.Description IsNot Nothing Then
			item.DescriptionText = ievent.Description.Value
		End If

		If ievent.Attendee IsNot Nothing Then
			For Each attendee As Cal_Address In ievent.Attendee
				Dim contactName As String = attendee.Value

				' Check if there is already a contact with that name in the schedule
				Dim contact As Contact = GetContact(contactName)

				' Check if the contact was found
				If contact Is Nothing Then
					' Create a new Contact object
					contact = New Contact()
					contact.Name = contactName

					' Attempt to parse the contactName
					If contactName.IndexOf(",") <> -1 Then
						Dim parts As String() = contactName.Split(New Char() {","c}, StringSplitOptions.RemoveEmptyEntries)
						If parts.Length > 1 Then
							contact.FirstName = parts(1).Trim()
							contact.LastName = parts(0).Trim()
						ElseIf parts.Length > 0 Then
							contact.FirstName = parts(0).Trim()
						End If
					Else
						Dim parts As String() = contactName.Split(New Char() {" "c}, StringSplitOptions.RemoveEmptyEntries)
						If parts.Length > 1 Then
							contact.FirstName = parts(0).Trim()
							contact.LastName = parts(1).Trim()
						ElseIf parts.Length > 0 Then
							contact.FirstName = parts(0).Trim()
						End If
					End If

					If attendee.EmailAddress IsNot Nothing Then
						contact.Email = attendee.EmailAddress
					End If

					' Add the contact to the schedule
					calendar.Schedule.Contacts.Add(contact)
				End If

				' Associate the item with this contact
				item.Contacts.Add(contact)
			Next
		End If

		' Transfer location
		If ievent.Location IsNot Nothing Then
			Dim locationName As String = ievent.Location.Value

			' Check if there is already a location with that name in the schedule
			Dim location As Location = GetLocation(locationName)

			' Check if the location was found
			If location Is Nothing Then
				' Create a new Location object
				location = New Location()
				location.Name = locationName

				' Add the location to the schedule
				calendar.Schedule.Locations.Add(location)
			End If

			' Associate the location with the appointment
			item.Location = location
		End If

		' Transfer reminder
		If ievent.Alarms.Count > 0 Then
			Dim ialarm As Alarm = ievent.Alarms(0)

			Dim reminder As New Reminder()
			If ialarm.Description IsNot Nothing Then
				reminder.Message = ialarm.Description
			End If

			' Check the type of the reminder
			If ialarm.Trigger.IsRelative Then
				' Check the relation type
				If ialarm.Trigger.Related = Trigger.TriggerRelation.START Then
					' Its relative to the start of the event
					reminder.Type = ReminderType.Leading
					reminder.TimeInterval = -ialarm.Trigger.Duration.Value
				Else
					' Its relative to the end of the event.
					' However, since MindFusion.Scheduling does not support
					' such kind of reminder relation, we will calculate
					' the same interval, relative to the start of the
					' item
					reminder.Type = ReminderType.Leading
					reminder.TimeInterval = ialarm.Trigger.Duration.Value + (item.EndTime - item.StartTime)
				End If
			Else
				' Its a fixed reminder
				reminder.Type = ReminderType.Exact
				reminder.Time = ialarm.Trigger.DateTime.Value
			End If

			' Associate the reminder with the item
			item.Reminder = reminder
		End If
	End Sub

	''' <summary>
	''' Copies all non-recurrent properties from the specified MindFusion.Scheduling
	''' item to the specified i-event.
	''' </summary>
	Private Sub CopyPropertiesNoRec(ByVal ievent As [Event], ByVal item As Item)
		' Transfer general properties
		ievent.Start = New Date_Time(item.StartTime)
		ievent.[End] = New Date_Time(item.EndTime)
		ievent.IsAllDay = item.AllDayEvent

		If item.HeaderText <> "" Then
			ievent.Summary = New DDay.iCal.DataTypes.Text(item.HeaderText)
		End If

		If item.DescriptionText <> "" Then
			ievent.Description = New DDay.iCal.DataTypes.Text(item.DescriptionText)
		End If

		' Transfer contacts
		If item.Contacts.Count > 0 Then
			Dim count As Integer = item.Contacts.Count
			Dim contactsArray As Cal_Address() = New Cal_Address(count - 1) {}
			For i As Integer = 0 To item.Contacts.Count - 1
				Dim contact As New Cal_Address()

				contact.Value = item.Contacts(i).FirstName + " " + item.Contacts(i).LastName

				contactsArray(i) = contact
			Next

			ievent.Attendee = contactsArray
		End If

		' Transfer location
		If item.Location IsNot Nothing Then
			ievent.Location = item.Location.Name
		End If

		' Transfer reminder
		If item.Reminder IsNot Nothing Then
			Dim alarm As New Alarm(ievent)
			alarm.Trigger = New Trigger()

			If item.Reminder.Type = ReminderType.Leading Then
				alarm.Trigger.Related = Trigger.TriggerRelation.START
				alarm.Trigger.Duration = New Duration(-item.Reminder.TimeInterval)
				alarm.Trigger.DateTime = Nothing
			Else
				alarm.Trigger.DateTime = New Date_Time(item.Reminder.Time)
				alarm.Trigger.Duration = Nothing
			End If

			ievent.Alarms.Add(alarm)
		End If
	End Sub

	''' <summary>
	''' Loads the ICalendar data contained in the specified file.
	''' </summary>
	Private Sub LoadICalendar(ByVal fileName As String)
		' Clear the current schedule
		calendar.Schedule.Clear()

		' Map between the uid's of recurrent items and
		' the actual MindFusion.Scheduling recurrence masters
		recurrentItems = New Hashtable()

		' A list of recurrence exceptions - processed
		' after all events are intially loaded
		Dim exceptions As New List(Of [Event])()

		' A list of deleted recurrence exceptions - the
		' id of the master item and the date of the deleted exception
		Dim deletedExceptions As New Hashtable()

		Dim icalendar__1 As iCalendar = iCalendar.LoadFromFile(fileName)
		For Each component As UniqueComponent In icalendar__1.UniqueComponents
			If Not (TypeOf component Is [Event]) Then
				Continue For
			End If

			Dim ievent As [Event] = TryCast(component, [Event])

			' Check first if the event is a recurrence exception.
			' The exception might come before the actual master item,
			' so handle this by processing exceptions at the end
			If ievent.Properties IsNot Nothing AndAlso ievent.Properties.Count > 0 Then
				If ievent.Properties.ContainsKey("RECURRENCE-ID") Then
					' Register for subsequent processing
					exceptions.Add(ievent)

					' Skip further processing of this event for now
					Continue For
				End If
			End If

			Dim app As New Appointment()

			' Transfer all properties, par recurrence
			CopyPropertiesNoRec(app, ievent)

			' Check for deleted exceptions
			If ievent.ExDate IsNot Nothing Then
				Dim deletedDates As New List(Of DateTime)()
				For Each [date] As RDate In ievent.ExDate
					If [date].Periods Is Nothing OrElse [date].Periods.Count = 0 Then
						Continue For
					End If

					deletedDates.Add([date].Periods(0).StartTime.Value)
				Next

				deletedExceptions(ievent.UID.Value) = deletedDates
			End If

			' Transfer recurrence
			If ievent.RRule IsNot Nothing Then
				Dim irule As Integer = 0
				For Each rule As Recur In ievent.RRule
					Dim recurrence As New Recurrence()
					recurrence.StartDate = ievent.DTStart.Value

					' The recurrence end type
					If rule.Count > 0 AndAlso rule.Count <> 2147483647 Then
						recurrence.NumOccurrences = rule.Count
						recurrence.RecurrenceEnd = RecurrenceEnd.NumOccurrences
					ElseIf rule.Until IsNot Nothing Then
						recurrence.EndDate = rule.Until.Value
						recurrence.RecurrenceEnd = RecurrenceEnd.EndDate
					Else
						recurrence.RecurrenceEnd = RecurrenceEnd.Never
					End If

					' The recurrence pattern
					If rule.Frequency = Recur.FrequencyType.DAILY Then
						If rule.ByDay Is Nothing OrElse rule.ByDay.Count = 0 Then
							recurrence.Pattern = RecurrencePattern.Daily
							recurrence.DailyRecurrence = DailyRecurrence.ByDayInterval
							recurrence.Days = rule.Interval
						ElseIf rule.ByDay(0).DayOfWeek = DayOfWeek.Monday Then
							recurrence.Pattern = RecurrencePattern.Daily
							recurrence.DailyRecurrence = DailyRecurrence.EveryWorkday
						Else
							recurrence.Pattern = RecurrencePattern.Daily
							recurrence.DailyRecurrence = DailyRecurrence.EveryWeekend
						End If
					ElseIf rule.Frequency = Recur.FrequencyType.WEEKLY Then
						recurrence.Pattern = RecurrencePattern.Weekly
						recurrence.Weeks = rule.Interval

						Dim daysOfWeek__2 As DaysOfWeek = DaysOfWeek.None
						For Each day As Recur.DaySpecifier In rule.ByDay
							daysOfWeek__2 = ToDaysOfWeek(day.DayOfWeek) Or daysOfWeek__2
						Next
						recurrence.DaysOfWeek = daysOfWeek__2
					ElseIf rule.Frequency = Recur.FrequencyType.MONTHLY Then
						recurrence.Pattern = RecurrencePattern.Monthly
						recurrence.Months = rule.Interval

						If rule.ByDay IsNot Nothing AndAlso rule.ByDay.Count > 0 Then
							recurrence.MonthlyRecurrence = MonthlyRecurrence.ByDayType
							For i As Integer = 0 To rule.ByDay.Count - 1
								recurrence.Day = ToDayOfWeekType(rule.ByDay(i).DayOfWeek)
								Select Case rule.ByDay(i).Num

									Case -1
										' Last
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Last
										Exit Select

									Case 1
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.First
										Exit Select

									Case 2
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Second
										Exit Select

									Case 3
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Third
										Exit Select

									Case 4
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Fourth
										Exit Select

								End Select

								' The first recurrence is assigned to the appointment being
								' loaded. Subsequent recurrences are assigned to clones of
								' this appointment, since MindFusion.Scheduling cannot support this
								' type of recurrence patterns
								If i = 0 Then
									app.Recurrence = TryCast(recurrence.Clone(), Recurrence)

									' Register the clone
									calendar.Schedule.Items.Add(app)

									' Register this item as a recurrence item under the same id
									RegisterRecurrenceMaster(ievent.UID.Value, app)
								Else
									' Create a clone of the current appointment and assign
									' a clone of the recurrence to it
									Dim clone As Appointment = TryCast(app.Clone(), Appointment)
									clone.Recurrence = TryCast(recurrence.Clone(), Recurrence)

									' Register the clone
									calendar.Schedule.Items.Add(clone)

									' Register this item as a recurrence item under the same id
									RegisterRecurrenceMaster(ievent.UID.Value, clone)
								End If
							Next

							' Skip further processing
							Continue For
						ElseIf rule.ByMonthDay IsNot Nothing AndAlso rule.ByMonthDay.Count > 0 Then
							recurrence.MonthlyRecurrence = MonthlyRecurrence.ByDayNumber
							For i As Integer = 0 To rule.ByMonthDay.Count - 1
								recurrence.DayOfMonth = rule.ByMonthDay(i)

								' The first recurrence is assigned to the appointment being
								' loaded. Subsequent recurrences are assigned to clones of
								' this appointment, since MindFusion.Scheduling cannot support this
								' type of recurrence patterns
								If i = 0 Then
									app.Recurrence = TryCast(recurrence.Clone(), Recurrence)

									' Register the clone
									calendar.Schedule.Items.Add(app)

									' Register this item as a recurrence item under the same id
									RegisterRecurrenceMaster(ievent.UID.Value, app)
								Else
									' Create a clone of the current appointment and assign
									' a clone of the recurrence to it
									Dim clone As Appointment = TryCast(app.Clone(), Appointment)
									clone.Recurrence = TryCast(recurrence.Clone(), Recurrence)

									' Register the clone
									calendar.Schedule.Items.Add(clone)

									' Register this item as a recurrence item under the same id
									RegisterRecurrenceMaster(ievent.UID.Value, clone)
								End If
							Next

							' Skip further processing
							Continue For
						Else
						End If
					ElseIf rule.Frequency = Recur.FrequencyType.YEARLY Then
						recurrence.Pattern = RecurrencePattern.Yearly
						If rule.ByDay IsNot Nothing AndAlso rule.ByDay.Count > 0 Then
							For i As Integer = 0 To rule.ByDay.Count - 1

								Select Case rule.ByDay(i).Num
									Case -1
										'Last
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Last
										Exit Select
									Case 1
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.First
										Exit Select
									Case 2
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Second
										Exit Select
									Case 3
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Third
										Exit Select
									Case 4
										recurrence.Occurrence = MindFusion.Scheduling.Occurrence.Fourth
										Exit Select
								End Select
								recurrence.MonthOfYear = rule.ByMonth(i)
								recurrence.DaysOfWeek = ToDaysOfWeek(rule.ByDay(i).DayOfWeek)

								'The first recurrence is assigned to the appointment being
								'loaded. Subsequent recurrences are assigned to clones of
								'this appointment, since MindFusion.Scheduling cannot support this
								'type of recurrence patterns
								If i = 0 Then
									app.Recurrence = DirectCast(recurrence.Clone(), Recurrence)

									'Register the clone
									calendar.Schedule.Items.Add(app)

									'Register this item as a recurrence item under the same id
									RegisterRecurrenceMaster(ievent.UID.Value, app)
								Else
									'Create a clone of the current appointment and assign
									'a clone of the recurrence to it
									Dim clone As Appointment = DirectCast(app.Clone(), Appointment)
									clone.Recurrence = DirectCast(recurrence.Clone(), Recurrence)

									'Register the clone
									calendar.Schedule.Items.Add(clone)

									'Register this item as a recurrence item under the same id
									RegisterRecurrenceMaster(ievent.UID.Value, clone)
								End If
							Next

							Continue For
						Else
							recurrence.YearlyRecurrence = YearlyRecurrence.SpecificDate
							recurrence.DayOfMonth = rule.ByMonthDay(0)
							recurrence.MonthOfYear = rule.ByMonth(0)
						End If
					End If

					' The first recurrence is assigned to the appointment being
					' loaded. Subsequent recurrences are assigned to clones of
					' this appointment, since MindFusion.Scheduling cannot support this
					' type of recurrence patterns
					If irule = 0 Then
						app.Recurrence = TryCast(recurrence, Recurrence)

						' Register the item
						calendar.Schedule.Items.Add(app)

						' Register the item if it is a recurrence master
						If ievent.RRule IsNot Nothing AndAlso ievent.RRule.Length > 0 Then
							If ievent.UID IsNot Nothing Then
								RegisterRecurrenceMaster(ievent.UID.Value, app)
							End If
						End If
					Else
						' Create a clone of the current appointment and assign
						' a clone of the recurrence to it
						Dim clone As Appointment = TryCast(app.Clone(), Appointment)
						clone.Recurrence = TryCast(recurrence.Clone(), Recurrence)

						' Register the clone
						calendar.Schedule.Items.Add(clone)

						' Register this item as a recurrence item under the same id
						RegisterRecurrenceMaster(ievent.UID.Value, clone)
					End If

					irule += 1
				Next
			Else
				' Register the clone
				calendar.Schedule.Items.Add(app)
			End If
		Next

		' Now that all events are loaded, process recurrence exceptions
		For Each ievent As [Event] In exceptions
			Dim [property] As [Property] = TryCast(ievent.Properties("RECURRENCE-ID"), [Property])
			Dim originalDate As New Date_Time([property].Value)

			' Get the recurrence master(s). It is possible to have
			' more than one master if the recurrence pattern cannot
			' be represented with a single recurrence in MindFusion.Scheduling
			Dim masters As List(Of Item) = TryCast(recurrentItems(ievent.UID.Value), List(Of Item))
			If masters Is Nothing Then
				Continue For
			End If

			If masters.Count = 0 Then
				Continue For
			End If

			' Attempt to generate occurrences for each recurrence in the list
			For Each master As Item In masters
				If master.Recurrence Is Nothing Then
					Continue For
				End If

				Dim items As ItemCollection(Of Item) = master.Recurrence.GenerateItems(originalDate.Value, originalDate.Value)

				If items.Count = 0 Then
					Continue For
				End If

				' Make the exception by setting the occurrence's properties
				CopyPropertiesNoRec(items(0), ievent)

				Exit For
			Next
		Next

		' Process deleted exceptions as well
		For Each uid As String In deletedExceptions.Keys
			' Get the recurrence master(s). It is possible to have
			' more than one master if the recurrence pattern cannot
			' be represented with a single recurrence in MindFusion.Scheduling
			Dim masters As List(Of Item) = TryCast(recurrentItems(uid), List(Of Item))
			If masters Is Nothing Then
				Continue For
			End If

			If masters.Count = 0 Then
				Continue For
			End If

			' Attempt to generate occurrences for each recurrence in the list
			For Each deletedDate As DateTime In TryCast(deletedExceptions(uid), List(Of DateTime))
				For Each master As Item In masters
					If master.Recurrence Is Nothing Then
						Continue For
					End If

					Dim items As ItemCollection(Of Item) = master.Recurrence.GenerateItems(deletedDate, deletedDate)

					If items.Count = 0 Then
						Continue For
					End If

					' Mark this exception as deleted
					master.Recurrence.MarkException(items(0), True)

					Exit For
				Next
			Next
		Next
	End Sub

	''' <summary>
	''' Saves the calendar in an ICalendar format in the specified file.
	''' </summary>
	Private Sub SaveICalendar(ByVal fileName As String)
		Dim icalendar As New iCalendar()

		icalendar.ProductID = "MindFusion.Scheduling Control"

		For Each item As Item In calendar.Schedule.Items
			Dim ievent As [Event] = icalendar.Create(Of [Event])()

			CopyPropertiesNoRec(ievent, item)

			If item.Recurrence IsNot Nothing Then
				Dim rule As New Recur()

				' The recurrence end type
				Select Case item.Recurrence.RecurrenceEnd

					Case RecurrenceEnd.EndDate
						rule.Until = New Date_Time(item.Recurrence.EndDate)
						Exit Select

					Case RecurrenceEnd.Never
						'2147483647 used as a workaround to handle recurrences that never end
						rule.Count = 2147483647
						rule.Until = Nothing
						Exit Select

					Case RecurrenceEnd.NumOccurrences
						rule.Count = item.Recurrence.NumOccurrences
						rule.Until = Nothing
						Exit Select

				End Select

				Select Case item.Recurrence.Pattern

					Case RecurrencePattern.Daily
						If True Then
							rule.Frequency = Recur.FrequencyType.DAILY

							Select Case item.Recurrence.DailyRecurrence

								Case DailyRecurrence.ByDayInterval
									If True Then
										rule.Interval = item.Recurrence.Days
									End If
									Exit Select

								Case DailyRecurrence.EveryWeekend
									If True Then
										rule.ByDay = New List(Of Recur.DaySpecifier)()
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Saturday))
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Sunday))
									End If
									Exit Select

								Case DailyRecurrence.EveryWorkday
									If True Then
										rule.ByDay = New List(Of Recur.DaySpecifier)()
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Monday))
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Tuesday))
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Wednesday))
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Thursday))
										rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Friday))
									End If
									Exit Select

							End Select
						End If
						Exit Select

					Case RecurrencePattern.Weekly
						If True Then
							rule.Frequency = Recur.FrequencyType.WEEKLY
							rule.Interval = item.Recurrence.Weeks

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Monday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Monday))
							End If

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Tuesday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Tuesday))
							End If

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Wednesday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Wednesday))
							End If

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Thursday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Thursday))
							End If

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Friday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Friday))
							End If

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Saturday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Saturday))
							End If

							If (item.Recurrence.DaysOfWeek And DaysOfWeek.Sunday) > 0 Then
								rule.ByDay.Add(New Recur.DaySpecifier(DayOfWeek.Sunday))
							End If
						End If
						Exit Select

					Case RecurrencePattern.Monthly
						If True Then
							rule.Frequency = Recur.FrequencyType.MONTHLY
							rule.Interval = item.Recurrence.Months

							Select Case item.Recurrence.MonthlyRecurrence

								Case MonthlyRecurrence.ByDayType
									If True Then
										Dim day As New Recur.DaySpecifier()

										day.DayOfWeek = FromDayOfWeekType(item.Recurrence.Day)

										Select Case item.Recurrence.Occurrence

											Case MindFusion.Scheduling.Occurrence.Last
												day.Num = -1
												Exit Select

											Case MindFusion.Scheduling.Occurrence.First
												day.Num = 1
												Exit Select

											Case MindFusion.Scheduling.Occurrence.Second
												day.Num = 2
												Exit Select

											Case MindFusion.Scheduling.Occurrence.Third
												day.Num = 3
												Exit Select

											Case MindFusion.Scheduling.Occurrence.Fourth
												day.Num = 4
												Exit Select

										End Select

										rule.ByDay.Add(day)
									End If
									Exit Select

								Case MonthlyRecurrence.ByDayNumber
									If True Then
										rule.ByMonthDay.Add(item.Recurrence.DayOfMonth)
									End If
									Exit Select

							End Select
						End If
						Exit Select

					Case RecurrencePattern.Yearly
						If True Then
							rule.Frequency = Recur.FrequencyType.YEARLY

							Select Case item.Recurrence.YearlyRecurrence
								Case YearlyRecurrence.ByDayType
									If True Then
										Dim day As New Recur.DaySpecifier()

										day.DayOfWeek = FromDayOfWeekType(item.Recurrence.Day)

										Select Case item.Recurrence.Occurrence

											Case MindFusion.Scheduling.Occurrence.Last
												day.Num = -1
												Exit Select

											Case MindFusion.Scheduling.Occurrence.First
												day.Num = 1
												Exit Select

											Case MindFusion.Scheduling.Occurrence.Second
												day.Num = 2
												Exit Select

											Case MindFusion.Scheduling.Occurrence.Third
												day.Num = 3
												Exit Select

											Case MindFusion.Scheduling.Occurrence.Fourth
												day.Num = 4
												Exit Select

										End Select

										rule.ByDay.Add(day)
										rule.ByMonth.Add(item.Recurrence.MonthOfYear)
										Exit Select
									End If
								Case YearlyRecurrence.SpecificDate
									If True Then
										rule.ByMonthDay.Add(item.Recurrence.DayOfMonth)
										rule.ByMonth.Add(item.Recurrence.MonthOfYear)
										Exit Select
									End If
							End Select
						End If
						Exit Select

				End Select

				' Mark deleted exceptions
				Dim deleted As New List(Of RecurrenceException)()
				For Each exception As RecurrenceException In item.Recurrence.Exceptions
					If exception.IsDeleted Then
						deleted.Add(exception)
					End If
				Next

				If deleted.Count > 0 Then
					Dim deletedExceptions As RDate() = New RDate(deleted.Count - 1) {}
					For i As Integer = 0 To deleted.Count - 1
						ievent.AddSingleException(New Date_Time(deleted(i).Item.StartTime))
					Next
				End If

				' Process non-deleted exceptions
				For Each exception As RecurrenceException In item.Recurrence.Exceptions
					If exception.IsDeleted Then
						Continue For
					End If

					' Create a new item representing the exception...
					Dim revent As [Event] = icalendar.Create(Of [Event])()

					CopyPropertiesNoRec(revent, exception.Item)

					Dim isDeleted As Boolean = False
					Dim original As Item = item.Recurrence.GetOccurrence(exception.Item.OccurrenceIndex, isDeleted, True)

					Dim dtserializer As New Date_TimeSerializer(New Date_Time(original.StartTime))

					Dim originalDate As String = dtserializer.SerializeToString()

					Dim [property] As New [Property](revent, "RECURRENCE-ID", originalDate)

					' Use the same UID as the master of the recurrence
					revent.UID = New DDay.iCal.DataTypes.Text(ievent.UID.Value)
				Next

				ievent.AddRecurrence(rule)
			End If
		Next

		' Perform the serialization
		Dim iserializer As New iCalendarSerializer()
		iserializer.iCalendar = icalendar
		iserializer.Serialize(fileName)
	End Sub
End Class
